{{

Spin code by Clock Loop.

The code communicates with the wx module, and the index.html caboose control webpage.
This will also communicate with another caller prop looking for /caboose path.
It will send the accelerometer values back to the engine prop. (or what ever caller)
As usual upload the index html to your wx module: Visit http://192.168.4.1/files/index.html
}}

CON 

  _CLKMODE  = XTAL1 + PLL16X                              ' The system clock spec
  _XINFREQ  = 5_000_000                                  ' Crystal

  Rx        = 31    'serial in from wx device
  Tx        = 30    'serial out to wx device
  
  RedLed    = 1    'Red Led      My custom pcb 
  BlueLed   = 10    'Blue Led     My custom pcb
  GreenLed  = 19     'Green Led    My custom pcb
                     
  EotLed    = 4     'End of train led pin.  
  Lantern   = 20    'Swinging lantern led pin.
                    
  AccXin    = 3     'X-axis PWM input.   Each axis has a 100 Hz PWM duty cycle output in which acceleration is proportional to the ratio tHx/Tx.
  AccYin    = 27    'Y-axis PWM input.   Each axis has a 100 Hz PWM duty cycle output in which acceleration is proportional to the ratio tHx/Tx.          
  
  'AccTemp   = 20    'Temperature (Analog Voltage) This pin is the buffered output of the temperature sensor.  
                    'The analog voltage at TOUT is an indication of the die temperature.  
                    'This voltage is useful as a differential measurement of temperature from ambient
                    ' and not as an absolute measurement of temperature. 
  
OBJ

    ser:    "Parallax Serial Terminal.spin"
    MM2125: "Memsic2125"

VAR 

    long  LedsHandle, YtiltHandle, XtiltHandle, CabooseHandle
    long  ReplyStatus
    long  XTiltListener, YTiltListener, LedsListener, CabooseListener
    
    long  Caboose1[50]
    long  Caboose2[50]
    long  Caboose3[50]


Pub Main      'Cog #1
  
  cognew(EOT, @Caboose1) ' Startup EOT device. Cog#2
  
  waitcnt(80_000_000 + cnt) 'Wait for 1 s                                                   

  cognew(Comms, @Caboose2)   'Cog #3 and #4
                               
  cognew(CabooseAccelerometer, @Caboose3)   'Start accelerometer loop. Cog #5 and #6
  
  Repeat
    TimeCounter  'Cog #1 
  
                         
PUB EOT                       'Flash EOT led on for 1 second then off for 1 second.
  outa[EotLed]  := 0
  dira[EotLed]~~              'Set led as output. 
                              '
  outa[Lantern]  := 0
  dira[Lantern]~~

  outa[RedLed]    := 1         'Turn Red Led off.    Common +
  outa[BlueLed]   := 1         'Turn Blue Led off.   Common +
  outa[GreenLed]  := 1         'Turn Green Led off.  Common +

  dira[RedLed]~~              'Set led as output. 
  dira[BlueLed]~~             'Set led as output.
  dira[GreenLed]~~            'Set led as output. 
                               
  
  
  Repeat 
    If LanternWeb == 0                                                                           
      outa[Lantern]  := 1         'Turn Lantern off.
    Else                             
      outa[Lantern]  := 0         'Turn Lantern off.
                                  
    If EotStatus == 0                                                                           
         
      outa[EotLed]  := 1         'Turn EOT on.
      
      If CabooseMove == 1
        waitcnt(80_000_000 + cnt) 'Wait for 1s.
      Else
        waitcnt(10_000_000 + cnt) 'Wait for 1s.
      
      outa[EotLed]  := 0         'Turn EOT off.
      
      If CabooseMove == 1
        waitcnt(80_000_000 + cnt) 'Wait for 1s.
      Else
        waitcnt(10_000_000 + cnt) 'Wait for 1s.                     
    
    Else
    
      outa[EotLed]  := 0         'Turn EOT off.

      outa[RedLed]    := 0         'Turn Red Led on.
      waitcnt(80_000_000 + cnt) 'Wait for 1s.
      outa[RedLed]    := 1         'Turn Red Led off.                    
        
      outa[BlueLed]   := 0         'Turn Blue Led on.
      waitcnt(80_000_000 + cnt) 'Wait for 1s.
      outa[BlueLed]   := 1         'Turn Blue Led off.
    
      outa[GreenLed]  := 0         'Turn Green Led on.
      waitcnt(80_000_000 + cnt) 'Wait for 1s.                             
      outa[GreenLed]  := 1         'Turn Green Led off.

Pub TimeCounter
  
  waitcnt(80_000_000 + cnt) 'Wait for 1s.  
  DataCycleTime++       
                                 
PUB CabooseAccelerometer | x, y
    
  MM2125.start(AccXin, AccYin)        ' Initialize Memsic 2125      Cog #4  
  waitcnt(80_000_000 + cnt) 'Wait for 1s. 
                             
  repeat    'Cog #1 loop
    x := MM2125.Mx                ' Read X axis
    y := MM2125.My                ' Read Y axis
    
    long[@TiltX][0] := x / 100
    long[@TiltY][0] := y / 100
       
    Xchange2 := Xchange1 - TiltX 
        
    Xchange1 := TiltX
        
    Ychange2 := Ychange1 - TiltY 
        
    Ychange1 := TiltY
    
    AxisChange := ||Xchange2 + ||Ychange2

    If TiltX == 0 OR TiltY == 0 OR AxisChange < 9 
          
      CabooseMove := 0
       
    Else
        
      CabooseMove := 1 

    WaitCnt(ClkFreq / 2 + Cnt)    'wait                                                     

Pub StartCommandMode
  
  CloseAllConnections
  
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module. 
  ser.stop
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module. 
  dira[TX]~~          ' Set direction to output
  outa[TX]:= 0        ' Set TX low.                         
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  dira[TX]~           ' Set direction to input  
  
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  ser.StartRxTx(RX, TX, 0, 115200)   'Cog #8  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.


    
Pub Comms
 
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  ser.StartRxTx(RX, TX, 0, 115200)   'Cog #8  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.
  
  StartCommandMode

  ser.char(13)                'End marker         


  ser.str(string(27))     'Set ansi command mode in putty.
  ser.str(string("[2J"))  'Clear putty telnet screen. 
  
  ser.str(string(27))           'Set ansi command mode in putty.                 
  ser.str(string("[1;3f"))      'Set Position in putty.      
  ser.str(string("Prop Started, Wx command mode Active.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0 
  
  ser.char(13)                'End marker         

  Repeat 10
    waitcnt(80_000_000 + cnt)   'wait 1/2 second before next update     

  StartListeners

'-------------------------------------------------------------------------------------------------------------------------------------------    
  PollError := 1      
  Repeat                'The main repeat loop.

        GetPoll   
            Case ReplyPoll[2]                     
            
              "G": 
                GetXtiltWeb
                GetYtiltWeb
                GetCabooseTiltInfo
                          
              "P": 
                GetLedsInfo
              
                
              "N":
                If LoopCounter1 > 15
                  PuttyTerminalOutput
                  LoopCounter1 := 0
              
              '"E":
                                                                    
        LoopCounter1++
                        
        If DataCycleTime > 50
          DataCycleTime := 0
    
          CloseAllConnections 
          waitcnt(10_000_000 + cnt)   'wait 1/2 second before next update   
          StartCommandMode
          waitcnt(10_000_000 + cnt)   'wait 1/2 second before next update     
          StartListeners    
          waitcnt(10_000_000 + cnt)   'wait 1/2 second before next update      
 
'-------------------------------------------------------------------------------------------------------------------------------------------    
Pub PuttyTerminalOutput

    ser.char(13)                'End marker  
    
    ser.str(string(27))     'Set ansi command mode in putty.                            
    ser.str(string("[2J"))  'Clear putty telnet screen.

    ser.str(string(27))     'Set ansi command mode in putty.
    ser.str(string("[2J"))  'Clear putty telnet screen. 
    
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[2;13f")) 'Set Position in putty.      
    ser.str(string("In Main Loop. ")) 'Tell putty command mode is on.                       
    
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[3;3f")) 'Set Position in putty.                                                       
    
    If ReplyStatus == 1     
      ser.str(string("Good signal."))   'If last reply was good, put it back up on putty which was cleared.
      ReplyStatus := 0                 'Set the reply to none so it can recheck for a new reply.                                                
    Else
      ser.str(string("  No signal."))    'If last reply was bad, put it back up on putty which was cleared.   

    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[5;3f")) 'Set Position in putty 
    ser.str(string("Tilt X that the Prop sent- ")) '
    ser.dec(long[@TiltX][0])
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[6;3f")) 'Set Position in putty 
    ser.str(string("Tilt Y that the Prop sent- ")) '
    ser.dec(long[@TiltY][0])
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[7;3f")) 'Set Position in putty  
    ser.str(string("End Of Train Light is- "))                                  
    If EotStatus == 0
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[8;3f")) 'Set Position in putty  
    ser.str(string("Lantern Light is- "))                                  
    
    If LanternWeb == 0
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
 
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[9;3f")) 'Set Position in putty  
    ser.str(string("Is cab00ze moving- "))                                  
    
    If CabooseMove == 0
      ser.str(string("No ")) '  
    Else
      ser.str(string("Yes"))
      
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[10;3f")) 'Set Position in putty 
    ser.str(string("Tilt X change- ")) '
    ser.dec(long[@Xchange2][0])
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[11;3f")) 'Set Position in putty 
    ser.str(string("Tilt Y change- ")) '
    ser.dec(long[@Ychange2][0])                            
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[12;3f")) 'Set Position in putty 
    ser.str(string("Time since sent data to engine- ")) '
    ser.dec(DataCycleTime)                          

    ser.char(13)    'End marker 
                                                                                                             
    ser.str(string(27))      'Set ansi command mode in putty.                                               
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  

    ser.char(13)                'End marker         


Pub StartListeners
  

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/TiltX"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 1   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "1" And Reply[5] == 13 
    XTiltListener := Reply[4]  
  Else

      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;13f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Xtlt listener returned.")) 'Tell putty error.                       
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  
      
      ser.char(13)                'End marker         

      
      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
      XTiltListener := 0
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/TiltY"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 2   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
    
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "2" And Reply[5] == 13 
    YTiltListener := Reply[4]   
  Else

      ser.char(13)                'End marker         

      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;13f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Ytlt listener returned.")) 'Tell putty error.                       
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  
      
      ser.char(13)                'End marker         

      
      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
      YTiltListener := 0                          
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
    
  ser.rxflush
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/leds"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
                            
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "3" And Reply[5] == 13 
    LedsListener := Reply[4]  
  Else
      
      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;13f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong switch listener returned.")) 'Tell putty error.                    
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0   
      
      ser.char(13)                'End marker         
      
      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
      LedsListener := 0
      
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
    
  ser.rxflush
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/cabooze"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 4   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
                             
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "4" And Reply[5] == 13 
    CabooseListener := Reply[4]  
  Else
      
      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;13f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Webcabose listener returned.")) 'Tell putty error.                    
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  

      ser.char(13)                'End marker         

      waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
    
      CabooseListener := 0
    
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
   
  ser.char(13)                'End marker          
      
  ser.str(string(27))      'Set ansi command mode in putty.                 
  ser.str(string("[2;3f")) 'Set Position in putty.      
  ser.str(string("Listeners Started.")) 'Tell putty command mode is on.                       
  ser.str(string(27))      'Set ansi command mode in putty.      
  ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  

  ser.char(13)                'End marker         
        
Pub GetPoll

        Reply[0] := 0  ' clear data
        Reply[1] := 0  ' clear data
        Reply[2] := 0  ' clear data
        Reply[3] := 0  ' clear data
        Reply[4] := 0  ' clear data
        Reply[5] := 0  ' clear data
        Reply[6] := 0  ' clear data
        Reply[7] := 0  ' clear data
        Reply[8] := 0  ' clear data
        Reply[9] := 0  ' clear data
        Reply[10] := 0  ' clear data
        Reply[11] := 0  ' clear data
        Reply[12] := 0  ' clear data
        Reply[13] := 0  ' clear data
        Reply[14] := 0  ' clear data
    
        ReplyPoll[0] := 0  ' clear data
        ReplyPoll[1] := 0  ' clear data
        ReplyPoll[2] := 0  ' clear data
        ReplyPoll[3] := 0  ' clear data
        ReplyPoll[4] := 0  ' clear data
        ReplyPoll[5] := 0  ' clear data
        ReplyPoll[6] := 0  ' clear data
        ReplyPoll[7] := 0  ' clear data
    
        ser.rxflush
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s Wait.
        ser.char(254)  'Begin marker
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s Wait.
        ser.str(string("POLL:"))   'Tell the module to listen to a ANY protocol/path.
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s Wait.
        ser.char(13)    'End marker
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
        ReplyPoll[0] := ser.rxcheck   ' should be 254
        ReplyPoll[1] := ser.rxcheck   ' should be =
        ReplyPoll[2] := ser.rxcheck   ' should be P,G,S,D
        ReplyPoll[3] := ser.rxcheck   ' should be ,
        ReplyPoll[4] := ser.rxcheck   ' should be 5 (this could change) The connection identifier to use for this request
        ReplyPoll[5] := ser.rxcheck   ' should be ,
        ReplyPoll[6] := ser.rxcheck   ' should be 1 or a 2. Use this returned listener id with PATH or CLOSE commands. 
                                  ' There are a maximum of four listeners available. 
                                  ' Issuing additional LISTEN commands when all four listeners are already established,
                                  '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
        ReplyPoll[7] := ser.rxcheck   ' should be 13   
        
       
Pub GetCabooseTiltInfo
        
        If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "G" And ReplyPoll[6] == CabooseListener And ReplyPoll[7] == 13
            ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
            PollError := 0
            
            CabooseHandle := ReplyPoll[4]  'Store connection identifier.
          
            Reply[0] := 0  ' clear data
            Reply[1] := 0  ' clear data
            Reply[2] := 0  ' clear data
            Reply[3] := 0  ' clear data
            Reply[4] := 0  ' clear data
            Reply[5] := 0  ' clear data
            Reply[6] := 0  ' clear data
            Reply[7] := 0  ' clear data                  
            ReplyPackage


Pub ReplyPackage

            ser.rxflush             
            waitcnt(5_000_000 + cnt)    'wait 1/16 second.
            ser.char(254)               'Begin marker
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.char(CabooseHandle)     'An active connection handle; returned by POLL
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.str(string(","))        'Required for a reply
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.str(string("200"))      'The desired HTTP response code for the reply.                         
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.str(string(","))        'Required for a reply   
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.str(string("38"))        'Tell the module how many bytes? (or is this digits?) the reply is.
            waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            ser.char(13)                'End marker                                        
            waitcnt(5_000_000 + cnt)    'wait 1/16 second.                              
            
'----------------------Begin Data Packet------------------------------------------------            
            
            ser.char(13)                '\r new line          1
            ser.char(10)                '\n line feed         2                     '
            ser.str(string("Xaxis>"))   'Send your data.      8
            ser.dec(TiltX)              '                    12
            ser.char(0)                'End marker           13
            ser.char(13)                '\r new line         14
            ser.char(10)                '\n line feed        15
            ser.str(string("Yaxis>"))   'Send your data.     21
            ser.dec(TiltY)              '                    25
            ser.char(0)                'End marker           26
            ser.char(13)                '\r new line         27
            ser.char(10)                '\n line feed        28                                  
            ser.str(string("Moves>"))   'Send your data.     34
            ser.dec(CabooseMove)        '                    35                   
            ser.char(0)                'End marker           36
            ser.char(13)                '\r new line         37
            ser.char(10)                '\n line feed        38
            
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
            ser.char(13)    'End marker
            waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
            
            DataCycleTime := 0

Pub GetXtiltWeb        
        If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "G" And ReplyPoll[6] == XTiltListener And ReplyPoll[7] == 13
            ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
            PollError := 0
            
            XtiltHandle := ReplyPoll[4]  'Store connection identifier.
          
            Reply[0] := 0  ' clear data
            Reply[1] := 0  ' clear data
            Reply[2] := 0  ' clear data
            Reply[3] := 0  ' clear data
            Reply[4] := 0  ' clear data
            Reply[5] := 0  ' clear data
            Reply[6] := 0  ' clear data
            Reply[7] := 0  ' clear data                  
            
            ser.rxflush             
            waitcnt(5_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(254)               'Begin marker
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTÃ¢â‚¬â„¢s name argument (in query or body) on connection handle.
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(XtiltHandle)            'An active connection handle; returned by POLL
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string(","))        'Required for a reply
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("200"))      'The desired HTTP response code for the reply.                         
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string(","))        'Required for a reply   
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("4"))        'Tell the module how many bytes? (or is this digits?) the reply is.
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(13)                'End marker                                        
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.dec(long[@TiltX][0])              'Send 1 byte (which is up to 4 characters (in decimal) (xxxx) of data to the module which then sends it to the webpage.                                                                '                                              
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(13)    'End marker
            waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
 
Pub GetYtiltWeb                                    '
        If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "G" And ReplyPoll[6] == YTiltListener And ReplyPoll[7] == 13
            ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
            PollError := 0
            
            YtiltHandle := ReplyPoll[4]  'Store connection identifier.
          
            Reply[0] := 0  ' clear data
            Reply[1] := 0  ' clear data
            Reply[2] := 0  ' clear data
            Reply[3] := 0  ' clear data
            Reply[4] := 0  ' clear data
            Reply[5] := 0  ' clear data
            Reply[6] := 0  ' clear data
            Reply[7] := 0  ' clear data                  
            
            ser.rxflush             
            waitcnt(5_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(254)               'Begin marker
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTÃ¢â‚¬â„¢s name argument (in query or body) on connection handle.
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(YtiltHandle)            'An active connection handle; returned by POLL
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string(","))        'Required for a reply
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("200"))      'The desired HTTP response code for the reply.                         
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string(","))        'Required for a reply   
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.str(string("4"))        'Tell the module how many bytes? (or is this digits?) the reply is.
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(13)                'End marker                                        
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.dec(long[@TiltY][0])    'Send 1 byte (which is up to 4 characters (in decimal) (xxxx) of data to the module which then sends it to the webpage.                             
            waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
            ser.char(13)    'End marker
            waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.

Pub GetLedsInfo              
  If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "P" And ReplyPoll[6] == LedsListener And ReplyPoll[7] == 13
    ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
    PollError := 0
    
    LedsHandle := ReplyPoll[4]  'Store connection identifier.
  
    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data                  
    
    ser.rxflush             
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.str(string("ARG:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.char(LedsHandle)     'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.str(string(","))      'Required for a reply
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.str(string("io"))  'The argument name to retrieve.   
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
    ser.char(13)                'End marker                                    
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Let buffer fill.
    Reply[0] := ser.rxcheck   ' should be 254
    Reply[1] := ser.rxcheck   ' should be =
    Reply[2] := ser.rxcheck   ' should be S
    Reply[3] := ser.rxcheck   ' should be ,
    Reply[4] := ser.rxcheck   ' should be p. Argument value  (this could change) -1 no reply
    Reply[5] := ser.rxcheck   ' should be i. Argument value  (this could change) -1 no reply.
    Reply[6] := ser.rxcheck   ' should be n. Argument value  (this could change) -1 no reply. 
    Reply[7] := ser.rxcheck   ' should be #. 0 to 6 Argument value  (this could change) -1 no reply
    Reply[8] := ser.rxcheck   ' should be s. Argument value  (this could change) -1 no reply.
    Reply[9] := ser.rxcheck   ' should be t. Argument value  (this could change) -1 no reply. 
    Reply[10] := ser.rxcheck  ' should be a. Argument value  (this could change) -1 no reply
    Reply[11] := ser.rxcheck  ' should be t. Argument value  (this could change) -1 no reply.
    Reply[12] := ser.rxcheck  ' should be e. Argument value  (this could change) -1 no reply.
    Reply[13] := ser.rxcheck  ' should be #. 1 or 0 Argument value  (this could change) -1 no reply.                                     ' 
    Reply[14] := ser.rxcheck  ' should be 13                                         
            
    If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "p" And Reply[5] == "i" And Reply[6] == "n" And Reply[14] == 13

      If Reply[7] == "0"       'The pin number. End of Train.
        If Reply[13] == "0"    'The state of the pin
          EotStatus := 0                                                                                                                         
        Else 
          EotStatus := 1
                                                                                                                         
      Elseif Reply[7] == "1"   'The pin number.  Rearlight
        If Reply[13] == "0"    'The state of the pin
          LanternWeb := 0                                                       
        Else 
          LanternWeb := 1                                                                 
                              
      Else
        Repeat
          
          waitcnt(80_000_000 + cnt) 'Wait for 1s forever.
          
          ser.char(13)                'End marker         
          
          ser.str(string(27))      'Set ansi command mode in putty.                 
          ser.str(string("[2;3f")) 'Set Position in putty.      
          ser.str(string("ERROR, wrong switch pin returned.")) 'Tell putty error.                    
          ser.str(string(27))      'Set ansi command mode in putty.      
          ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0   
         
          ser.char(13)                'End marker         
                                  
      ser.char(254)               'Begin marker
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string("REPLY:")) 'Tell the module to send a reply to the poll response.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.char(LedsHandle)     'Should be a number.   An active connection handle; returned by POLL.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string("200"))    'The desired HTTP response code for the reply.                         
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string("2"))      'Tell the module how many bytes? (or is this digits?) the reply is.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.char(13)                'End marker               
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.str(string("OK"))     'Send OK to the webpage.                          
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s Wait.
      ser.char(13)    'End marker
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
      

Pub CloseHandles

  CloseCabooseHandle
  CloseLedsHandle
  CloseXtiltHandle
  CloseYtiltHandle
 

Pub CloseCabooseHandle

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(CabooseHandle)     'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      CabooseHandle := 0


Pub CloseLedsHandle

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(LedsHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      LedsHandle := 0 


Pub CloseXtiltHandle

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(XtiltHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      XtiltHandle := 0

Pub CloseYtiltHandle

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(YtiltHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      YtiltHandle := 0

Pub CloseAllConnections
      

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(CabooseHandle)     'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
           
      CabooseHandle := 0 
            
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(CabooseListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
            
      
      CabooseListener := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(LedsHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      LedsHandle := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(LedsListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      LedsListener := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(XtiltHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      XtiltHandle := 0
      
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(XTiltListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      XTiltListener := 0
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(YtiltHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      YtiltHandle := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(YTiltListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      YTiltListener := 0          
      
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("1"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("2"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("3"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("4"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("5"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("6"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("7"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("8"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.

      
DAT
  LoopCounter1  long  0
  PollError     byte  0
  
  DataCycleTime long  0
  AxisChange    long  0
  
  Xchange1      long  0
  Xchange2      long  0

  Ychange1      long  0
  Ychange2      long  0
  
  CabooseMove   byte  0
  
  TiltX         long  1234, 0
  TiltY         long  5678, 0
  EotStatus     byte  0
  LanternWeb    byte  0
  'Byte[#]           '0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 18 20
  Reply         byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  ReplyPoll     byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////      
}}   