{{

Spin code by Clock Loop, it only controls the motor and gets accelerometer values from the caboose.

The code communicates with the wx module, and the index.html motor control webpage.
As usual upload the index html to your wx module: Visit http://192.168.4.1/files/index.html
}}

CON 

  _CLKMODE=XTAL1 + PLL16X                              ' The system clock spec
  _XINFREQ = 5_000_000                                  ' Crystal

  Rx        = 31     'serial in from wx device
  Tx        = 30     'serial out to wx device
  
  RxEngineB = 15     'serial in from wx device
  TxEngineB = 14     'serial out to wx device
  
  RearLight = 0      'Pin output for rearlight led
 
  HeadLight = 1      'Pin output for headlight led
  
  Smoker    = 2      'Pin output for smoke maker
  
  AxleSense = 3      ' High means sensor not active.  Sensor goes low twice per revolution of the axle.
               
  INV       = 4      ' A logical high value inverts the meaning of IN1 and IN2 for both motor drivers 
                      
  SLEW      = 5      ' Output slew rate selection input. A logical LOW results in a slow output rise time (1.5 us - 6 us). 
                     ' A logical HIGH selects a fast output rise time (0.2 us - 1.45 us).
                     ' This pin should be set HIGH for high-frequency (over 10 kHz) PWM. This pin determines the slew rate mode for both motor driver ICs. 
                      
  EN        = 6      ' Enable input: when EN is LOW, both motor ICs are in a low-current sleep mode.       
  
  M1SF      = 7      ' Status flag output: an over-current (short circuit) or over-temperature event will cause SF to be latched LOW. 
                     ' If either of the disable pins (D1 or D2) are disabling the outputs, SF will also be LOW.
                     '  Otherwise, this pin is weakly pulled high. 
                     ' This allows the two SF pins on the board to be tied together and connected to a single MCU inpu.

  M1D1      = 8      ' Disable input: when D1 is high, OUT1 and OUT2 are set to high impedance. 
                     ' A D1 PWM duty cycle of 70% gives a motor duty cycle of 30%. 
                     ' Typically, only one of the two disable pins is used, but the default is for both disable pins to be active.  
  
  M1D2      = 9      ' Disable input: when D1 is high, OUT1 and OUT2 are set to high impedance.
                     ' A D1 PWM duty cycle of 70% gives a motor duty cycle of 30%. 
                     ' Typically, only one of the two disable pins is used, but the default is for both disable pins to be active

  M1IN1     = 10     ' The logic input control of OUT1. PWM can be applied to this pin (typically done with both disable pins inactive)
                     
  M1IN2     = 11     ' The logic input control of OUT2. PWM can be applied to this pin (typically done with both disable lines inactive)

  VDD       = 12     ' VDD   Only used for pullup resistors, safe to control with prop, NEVER connect this to VIN.

  M1fbDI    = 18     '(Feedback of current draw - not used in this basic demo)
  M1fbDO    = 19     '(Feedback of current draw - not used in this basic demo) 
  M1fbSCL   = 20     '(Feedback of current draw - not used in this basic demo) 
  M1fbCS    = 21     '(Feedback of current draw - not used in this basic demo)

  lPin      = 26     'left audio output
  rPin      = 27     'right audio output

  doPin     = 22     'sd card DO pin
  clkPin    = 23     'sd card CLK pin
  diPin     = 24     'sd card DI pin
  csPin     = 25     'sd card CS pin

  wpPin     = -1     'Needed for sound code - unused
  cdPin     = -1     'Needed for sound code - unused

  max_duty = 40     'My motor goes too fast higher.   These drastically change when using slew 1
  min_duty = 10      'My motor dosen't turn lower.    These drastically change when using slew 1
  MotorFreq = 1000   'Lower than 10k is a more quiet motor.    These drastically change when slew out is 1 (high)  


OBJ

    ser: "Parallax Serial Terminal.spin"
    
    serEngineB: "Parallax Serial Terminal.spin"

    pwm1  :  "pwmasm"
    
    pwm2  :  "pwmasm"

VAR 

    long  CabooseHandle, PwmHandle, MotorHandle, LedsHandle, CabooseWebHandle
    long  ReplyStatus, ReplyStatusEngineB
    long  MotorListener, LedsListener, PwmListener, CabooseListener, CabooseWebListener
    
    
    
    long  SmokeStack1[50]
    long  SmokeStack2[50]
    long  SmokeStack3[50]

Pub Main      'Cog #1

  cognew(Comms, @SmokeStack1)   'Cog #2, 3
  waitcnt(10_000_000 + cnt) 'Wait for 1/8 s                        
 
  cognew(CommsCaboose, @SmokeStack2)   'Cog #4, 5
  waitcnt(10_000_000 + cnt) 'Wait for 1/8 s                           
  
  cognew(ClockTimer, @SmokeStack3)   'Cog #6
  waitcnt(10_000_000 + cnt) 'Wait for 1/8 s
                               
  MotorController    'Start motor drive.  (Cog #1, 7, 8)

PUB ClockTimer
  
  dira[Headlight]~~    ' Set direction to output (Forward LED Light)
  dira[Rearlight]~~    ' Set direction to output (Reverse LED Light)
  dira[Smoker]~~       ' Set direction to output (Smoke Maker)  
  
  
  Repeat
    SetSwitches
    CheckTime         
    waitcnt(10_000_000 + cnt) 'Wait for 1 s
    
    Micro++
    If Micro > 7      
      Micro := 0    
      CabooseConnect++ 
      Seconds++
    
      If Seconds > 59    
        Minutes++
        Seconds := 0
        MinuteChime := 1  
            
  
        If Minutes > 59
          Hours++
          Minutes := 0      
          HourChime := 1
          
          If Hours > 12
             Hours := 1 
    
PUB MotorController
 
  
  dira[M1IN1]~~        ' Set direction to output (Controls for Motor 1)
  dira[M1IN2]~~        ' Set direction to output (Controls for Motor 1)  
  dira[M1D1]~~         ' Set direction to output (Controls for Motor 1) 
  dira[M1D2]~~         ' Set direction to output (Controls for Motor 1)
  dira[SLEW]~~         ' Set direction to output (Slew Controls for Motor 1 and 2)
  
  dira[EN]~~           ' Set direction to output (Enable Carrier Board)

  outa[Slew]    := 1   ' Set Pin SLEW Output slew rate selection input. A logical LOW results in a slow output rise time (1.5 us - 6 us). 
                       ' A logical HIGH selects a fast output rise time (0.2 us - 1.45 us).
                       ' This pin should be set HIGH for high-frequency (over 10 kHz) PWM. This pin determines the slew rate mode for both motor driver ICs.
                       ' Setting this to a 1 with a duty of 1000 also works.

  outa[EN]      := 1   ' Set Pin (EN)to high to enable carrier board
                         
  
  'dira[VDD]~~         ' Set direction to output (Enable Carrier Board) 
                                                                         
  'outa[VDD]     := 1  ' Set Pin VDD to high to enable PULL up resistors, NEVER connect this to VIN.  
                       ' 

       
  'outa[M1IN1] := 0    ' The logic input control of OUT2. PWM can be applied to this pin (typically done with both disable lines inactive)  Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction
                                    
  'outa[M1IN2] := 1    ' The logic input control of OUT1. PWM can be applied to this pin (typically done with both disable pins inactive)   Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction

  'outa[M1D1]  := 1    ' Disable input: when D1 is high, OUT1 and OUT2 are set to high impedance. 
                       ' A D1 PWM duty cycle of 70% gives a motor duty cycle of 30%. 
                       ' Typically, only one of the two disable pins is used, but the default is for both disable pins to be active.
                                             
  'outa[M1D2]  := 1    ' Disable input: when D1 is high, OUT1 and OUT2 are set to high impedance. 
                       'A D1 PWM duty cycle of 70% gives a motor duty cycle of 30%. 
                       'Typically, only one of the two disable pins is used, but the default is for both disable pins to be active
    

  pwm1.start(M1D1) 'Cog #4
  pwm2.start(M1D2) 'Cog #5
  
  pwm1.SetPeriod(MotorFreq)
  pwm2.SetPeriod(MotorFreq)
  
  'Set the initial direction to forward.
  outa[M1IN1] := 1     ' The logic input control of OUT1. PWM can be applied to this pin (typically done with both disable lines inactive)  Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction
  outa[M1IN2] := 0     ' The logic input control of OUT2. PWM can be applied to this pin (typically done with both disable pins inactive)   Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction  
        
  Repeat   'Cog #1
    MotorPwm     

                                                                                                          
Pub MotorPwm

  
    waitcnt(40_000_000 + cnt)   'wait 1/2 second before next update          
   
    If Speed > 98
      Speed := 99
           
    If Speed < 2
      Speed := 1 
        
    If CurrentDuty > 98
      CurrentDuty := 99
           
    If CurrentDuty < 1
      CurrentDuty := 1  
    
    CurrentSpeed := CurrentDuty 
    
    Repeat CurrentDuty from CurrentSpeed to Speed 'linearly advance speed 
   
          If FastStart == 1
            CurrentDuty := Speed
                                                             
          If CurrentDuty > 98
            CurrentDuty := 99
            
          If CurrentDuty < 1
            CurrentDuty := 1            
            
          If EmergencyStop == 1
            CurrentDuty := 1
            Speed := 1 
            pwm1.SetDuty(0)
            pwm2.SetDuty(0)

          pwm1.SetDuty(100-CurrentDuty)
          pwm2.SetDuty(CurrentDuty)

          waitcnt(40_000_000 + cnt)   'wait 1/2 second before next update          
           
    
Pub CheckTime


  If MinuteChime == 1
    QuarterMinute++
    FiveMinute++
    
    MinuteChime := 0 
    
    If FiveMinute > 4            
      
      FiveMinute := 0
      
      
      If TimerSwitch == 1        
        HeadlightSwitch := 1
        RearlightSwitch := 1      
        Speed := TimedSpeed
        Started := 1   
         
  If FiveMinute == 0 AND Seconds > SecondsModifier AND Started == 1 AND TimerSwitch == 1 
      Speed := 1
      Started := 0      

  If FiveMinute == 4 AND Seconds > 49 AND Started == 0 AND TimerSwitch == 1   
    !HeadlightSwitch
    !RearlightSwitch
    
    
  If TimerSwitch == 1 And FiveMinute < 4 And Started == 0 AND CurrentDuty < 3
    HeadlightSwitch := 0
    RearlightSwitch := 0
    
  
  If currentduty > 50 AND CabooseMove == 1
      TrainMoving := 1
      TrainMovingCount := 0
  
  ElseIf currentduty > 50 AND CabooseMove == 0
      If Micro == 0
        TrainMovingCount++
      TrainMoving := 0
  Else
    TrainMoving := 0
    TrainMovingCount := 0
                
  If TrainMovingCount > 10 AND EotStop == 1
        Speed := 0
      
  If TrainMovingCount > 10 AND CurrentDuty < 9 AND EotStop == 1
      TrainMovingCount := 0  
      TrainMoving := 0 
      EmergencyStop := 1 
  
    
Pub SetSwitches                 

    If EmergencyStop == 1
      
      If Micro == 0
        !HeadlightSwitch
      If Micro == 7 
        !RearlightSwitch
      CurrentDuty := 1
      Speed := 1
      pwm1.SetDuty(0)
      pwm2.SetDuty(0)
    
    If Reverse == 1
      If Micro == 0
        !RearlightSwitch
      If EmergencyStop == 0
        HeadlightSwitch := PrevHeadlightSwitch
             
    If Reverse == 0 And EmergencyStop == 0 And TimerSwitch == 0
      HeadlightSwitch := PrevHeadlightSwitch
      RearlightSwitch := PrevRearlightSwitch
         
    If Reverse == 0 And EmergencyStop == 1 And CurrentDuty < 3       ' Going Forward
      outa[M1IN1] := 1     ' The logic input control of OUT1. PWM can be applied to this pin (typically done with both disable lines inactive)  Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction
      outa[M1IN2] := 0     ' The logic input control of OUT2. PWM can be applied to this pin (typically done with both disable pins inactive)   Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction  
        
    ElseIf Reverse == 1 And EmergencyStop == 1 And CurrentDuty < 3   ' Going Reverse
      outa[M1IN1] := 0     ' The logic input control of OUT1. PWM can be applied to this pin (typically done with both disable lines inactive)  Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction
      outa[M1IN2] := 1     ' The logic input control of OUT2. PWM can be applied to this pin (typically done with both disable pins inactive)   Reversing the setings on Pins M1IN1 & M1IN2 will reverse motor direction      
        
    If HeadlightSwitch == 1
      outa[Headlight] := 1
    Else  
      outa[Headlight] := 0
      
    If RearlightSwitch == 1
      outa[Rearlight] := 1
    Else
      outa[Rearlight] := 0
    
    If SmokerSwitch == 1
      outa[Smoker] := 1 
    Else
      outa[Smoker] := 0       
    
    
Pub CommsCaboose

    waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
    serEngineB.StartRxTx(RxEngineB, TxEngineB, 0, 115200)   'Cog #3  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
    waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.

    StartCommandmodeEngineB

    ConnectErrorEngineB := 1
    
    Repeat
        

          Repeat while ConnectErrorEngineB == 1
               
            ConnectToCaboose
            
            If ConnectErrorEngineB == 1
              CloseCaboose            

            Else
              CabooseConnect := 0
          
          RequestCabooseData 
            
          Repeat while ConnectErrorEngineB == 0
    
              GetPollEngineB            
              If ReplyPollEngineB[2] == "D"
                GetDataFromCaboose            

                ConnectErrorEngineB := 1

              If CabooseConnect > 10
                ConnectErrorEngineB := 1 
                CloseAllConnectionsEngineB
                   

Pub Comms

  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  ser.StartRxTx(RX, TX, 0, 115200)   'Cog #3  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.
  
  
  
  StartCommandmode 
  
  ser.char(13)                'End marker         
  
  ser.str(string(27))           'Set ansi command mode in putty.                
  ser.str(string("[2;16f"))      'Set Position in putty.      
  ser.str(string("Prop Started.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0  

  ser.char(13)                'End marker         
   
  Repeat 10
    waitcnt(80_000_000 + cnt)   'wait 1/8 second for serial to start.

  
'-------------------------------------------------------------------------------------------------------------------------------------------    
  RestartError := 1 
  PropStartup := 0
  PollError := 1
         
  Repeat                    'The main repeat loop, Cog#2.
        
        Repeat while RestartError == 1 
          ListenError := 1

          StartCommandmode

          StartListeners

          RestartError := ListenError                            
          PropStartup := 1
            
        If PropStartup == 1
                
            LoopCounter1++
                               
            GetPoll   
            Case ReplyPoll[2]                     
            
              "G": 
                GetCabooseInfo               
                GetPwmInfo  
                                          
              "P": 
                GetLedsInfo
                GetMotorInfo             
                          
              "N":
                If LoopCounter1 > 15
                  PuttyTerminalOuput
                  LoopCounter1 := 0     


Pub StartCommandmode

  CloseCommandMode

  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  ser.Stop
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  dira[TX]~~          ' Set direction to output
  outa[TX]:= 0        ' Set TX low.                         
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  dira[TX]~           ' Set direction to input  
  
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  ser.StartRxTx(RX, TX, 0, 115200)   'Cog #3  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.

  ser.char(13)                'End marker         
  
  ser.str(string(27))     'Set ansi command mode in putty.
  ser.str(string("[2J"))  'Clear putty telnet screen. 
  
  ser.str(string(27))           'Set ansi command mode in putty.                 
  ser.str(string("[1;3f"))      'Set Position in putty.      
  ser.str(string("Wx command mode Active.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0          
 
  ser.char(13)                'End marker         
  
Pub StartCommandmodeEngineB

  CloseCommandModeEngineB
  
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  serEngineB.Stop
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  dira[TXEngineB]~~          ' Set direction to output
  outa[TXEngineB]:= 0        ' Set TX low.                         
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  dira[TXEngineB]~           ' Set direction to input  
  
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for command mode to start in the module.
  serEngineB.StartRxTx(RXEngineB, TXEngineB, 0, 115200)   'Cog #3  Parallax Serial Terminal does NOT work at receiving data 230400 for some reason.
  waitcnt(10_000_000 + cnt)   'wait 1/8 second for serial to start.
                   
Pub CloseCommandMode
      
      StopListeners
      
      CloseAllConnections
        
      Reply[0] := 0  ' clear data
      Reply[1] := 0  ' clear data
      Reply[2] := 0  ' clear data
      Reply[3] := 0  ' clear data
      Reply[4] := 0  ' clear data
      Reply[5] := 0  ' clear data
      Reply[6] := 0  ' clear data
      Reply[7] := 0  ' clear data
      
      ser.rxflush                     
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("SET:cmd-enable,0"))    'Tell the module turn off command mode.   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
 
                               
Pub CloseCommandModeEngineB
      
      CloseAllConnectionsEngineB
        
      ReplyEngineB[0] := 0  ' clear data
      ReplyEngineB[1] := 0  ' clear data
      ReplyEngineB[2] := 0  ' clear data
      ReplyEngineB[3] := 0  ' clear data
      ReplyEngineB[4] := 0  ' clear data
      ReplyEngineB[5] := 0  ' clear data
      ReplyEngineB[6] := 0  ' clear data
      ReplyEngineB[7] := 0  ' clear data
      
      serEngineB.rxflush                     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("SET:cmd-enable,0"))    'Tell the module to connect to a specific address.   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.                           

     
Pub PuttyTerminalOuput

    ser.char(13)                'End marker  

    ser.str(string(27))     'Set ansi command mode in putty.
    ser.str(string("[2J"))  'Clear putty telnet screen. 
                            '
    ser.str(string(27))     'Set ansi command mode in putty.
    ser.str(string("[2J"))  'Clear putty telnet screen. 
                            '    
    ser.str(string(27))      'Set ansi command mode in putty.                 
    ser.str(string("[2;13f")) 'Set Position in putty.      
    ser.str(string("In Main Loop. ")) 'Tell putty command mode is on.                       
    ser.str(string(27))      'Set ansi command mode in putty.      
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0

    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[4;3f")) 'Set Position in putty
                                                           
    If ReplyStatus == 1     
      ser.str(string("Good signal."))   'If last reply was good, put it back up on putty which was cleared.
      ReplyStatus := 0                 'Set the reply to none so it can recheck for a new reply.                                                
    Else
      ser.str(string("  No signal."))    'If last reply was bad, put it back up on putty which was cleared.   

    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[6;3f")) 'Set Position in putty 
    ser.str(string("WebSpeed that the webpage sent- ")) '
    ser.dec(Speed)
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[7;3f")) 'Set Position in putty 
    ser.str(string("CurrentDuty- ")) '
    ser.dec(CurrentDuty)
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[9;3f")) 'Set Position in putty  
    ser.str(string("Emergency Brake is- "))                                  
    If EmergencyStop == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[11;3f")) 'Set Position in putty 
    ser.str(string("Direction is- "))                                  
    If Reverse == 0
      ser.str(string("Forward")) '  
    Else
      ser.str(string("Reverse")) '
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[12;3f")) 'Set Position in putty 
    ser.str(string("Fast start is- ")) '
    If FastStart == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) 

    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[13;3f")) 'Set Position in putty 
    ser.str(string("StartTimer is- ")) '
    If TimerSwitch == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off"))
          
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[14;3f")) 'Set Position in putty 
    ser.str(string("Eot Stop is- ")) '
    If EotStop == 1
      ser.str(string("On ")) ' 
    Else
      ser.str(string("Off"))
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[16;3f")) 'Set Position in putty 
    ser.str(string("Headlight is- ")) '                                 
    If HeadlightSwitch == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
                             
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[17;3f")) 'Set Position in putty
    ser.str(string("Rearlight is- ")) '
    If RearlightSwitch == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[18;3f")) 'Set Position in putty
    ser.str(string("Smoker is- ")) '
    If SmokerSwitch == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '    
     
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[19;3f")) 'Set Position in putty 
    ser.str(string("Whistle is- ")) '                                 
    If Whistle == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
                             
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[20;3f")) 'Set Position in putty
    ser.str(string("Bell is- ")) '
    If Bell == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off")) '
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[21;3f")) 'Set Position in putty 
    ser.str(string("StackTalk is- ")) '
    If StackTalk == 1
      ser.str(string("On ")) '  
    Else
      ser.str(string("Off"))  
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[23;3f")) 'Set Position in putty 
    ser.str(string("TrainMoving:")) '       
    ser.dec(TrainMoving)
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[24;3f")) 'Set Position in putty 
    ser.str(string("TrainMovingCount:")) '
    ser.dec(TrainMovingCount) 
                                                        
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[26;3f")) 'Set Position in putty 
    ser.str(string("Is cab00se moving- ")) '
    If CabooseMove == 1
      ser.str(string("Yes")) '  
    Else
      ser.str(string("No ")) 
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[27;3f")) 'Set Position in putty 
    ser.str(string("TiltX- ")) '
    ser.dec(TiltX)
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[28;3f")) 'Set Position in putty 
    ser.str(string("TiltXchange- ")) '
    ser.dec(Xchange2)
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[29;3f")) 'Set Position in putty
    ser.str(string("TiltY- ")) '
    ser.dec(TiltY)    
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[30;3f")) 'Set Position in putty 
    ser.str(string("TiltYchange- ")) '
    ser.dec(Ychange2)       
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[31;3f")) 'Set Position in putty 
    ser.str(string("Kab00se says its- ")) '
    If Moves == 1
      ser.str(string("moving.    ")) '  
    Elseif Moves == 0 
      ser.str(string("not moving."))
    Else
      ser.str(string("no comms.  ")) 
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[32;3f")) 'Set Position in putty 
    ser.str(string("Time since last Kab00se conexction- ")) '
    ser.dec(CabooseConnect)  
    
    ser.str(string(27))      'Set ansi command mode in putty.                                             
    ser.str(string("[34;3f")) 'Set Position in putty 
    ser.str(string("Time-")) '
    ser.dec(Hours)   
    ser.str(string("*")) '
    ser.dec(Minutes)
    ser.str(string(".")) '
    ser.dec(Seconds)   
    
    ser.char(13)                'End marker         
    
    ser.str(string(27))      'Set ansi command mode in putty.                                               
    ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0 

    ser.char(13)    'End marker


    
'------------------------------------------------------------------------------------------------------------------------------------------- 
'    
Pub GetPoll

        Reply[0] := 0  ' clear data
        Reply[1] := 0  ' clear data
        Reply[2] := 0  ' clear data
        Reply[3] := 0  ' clear data
        Reply[4] := 0  ' clear data
        Reply[5] := 0  ' clear data
        Reply[6] := 0  ' clear data
        Reply[7] := 0  ' clear data
        Reply[8] := 0  ' clear data
        Reply[9] := 0  ' clear data
        Reply[10] := 0  ' clear data
        Reply[11] := 0  ' clear data
        Reply[12] := 0  ' clear data
        Reply[13] := 0  ' clear data
        Reply[14] := 0  ' clear data
        
        ReplyPoll[0] := 0  ' clear data
        ReplyPoll[1] := 0  ' clear data
        ReplyPoll[2] := 0  ' clear data
        ReplyPoll[3] := 0  ' clear data
        ReplyPoll[4] := 0  ' clear data
        ReplyPoll[5] := 0  ' clear data
        ReplyPoll[6] := 0  ' clear data
        ReplyPoll[7] := 0  ' clear data
        
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s   Wait
        ser.rxflush
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s   Wait
        ser.char(254)  'Begin marker
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s   Wait
        ser.str(string("POLL:"))   'Tell the module to listen to a ANY protocol/path.
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Wait
        ser.char(13)    'End marker
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
        ReplyPoll[0] := ser.rxcheck   ' should be 254
        ReplyPoll[1] := ser.rxcheck   ' should be =
        ReplyPoll[2] := ser.rxcheck   ' should be P
        ReplyPoll[3] := ser.rxcheck   ' should be ,
        ReplyPoll[4] := ser.rxcheck   ' should be 5 (this could change) The connection identifier to use for this request
        ReplyPoll[5] := ser.rxcheck   ' should be ,
        ReplyPoll[6] := ser.rxcheck   ' should be 1 or a 2. Use this returned listener id with PATH or CLOSE commands. 
                                      ' There are a maximum of four listeners available. 
                                      ' Issuing additional LISTEN commands when all four listeners are already established,
                                      '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
        ReplyPoll[7] := ser.rxcheck   ' should be 13 


Pub GetPollEngineB
        
        ReplyEngineB[0] := 0  ' clear data
        ReplyEngineB[1] := 0  ' clear data
        ReplyEngineB[2] := 0  ' clear data
        ReplyEngineB[3] := 0  ' clear data
        ReplyEngineB[4] := 0  ' clear data
        ReplyEngineB[5] := 0  ' clear data
        ReplyEngineB[6] := 0  ' clear data
        ReplyEngineB[7] := 0  ' clear data
        ReplyEngineB[8] := 0  ' clear data
        ReplyEngineB[9] := 0  ' clear data
        ReplyEngineB[10] := 0  ' clear data
        ReplyEngineB[11] := 0  ' clear data
        ReplyEngineB[12] := 0  ' clear data
        ReplyEngineB[13] := 0  ' clear data
        ReplyEngineB[14] := 0  ' clear data
        
        ReplyPollEngineB[0] := 0  ' clear data
        ReplyPollEngineB[1] := 0  ' clear data
        ReplyPollEngineB[2] := 0  ' clear data
        ReplyPollEngineB[3] := 0  ' clear data
        ReplyPollEngineB[4] := 0  ' clear data
        ReplyPollEngineB[5] := 0  ' clear data
        ReplyPollEngineB[6] := 0  ' clear data
        ReplyPollEngineB[7] := 0  ' clear data
        
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s   Wait
        serEngineB.rxflush
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s   Wait
        serEngineB.char(254)  'Begin marker
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s   Wait
        serEngineB.str(string("POLL:"))   'Tell the module to listen to a ANY protocol/path.
        waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Wait
        serEngineB.char(13)    'End marker
        waitcnt(5_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
        ReplyPollEngineB[0] := serEngineB.rxcheck   ' should be 254
        ReplyPollEngineB[1] := serEngineB.rxcheck   ' should be =
        ReplyPollEngineB[2] := serEngineB.rxcheck   ' should be P
        ReplyPollEngineB[3] := serEngineB.rxcheck   ' should be ,
        ReplyPollEngineB[4] := serEngineB.rxcheck   ' should be 5 (this could change) The connection identifier to use for this request
        ReplyPollEngineB[5] := serEngineB.rxcheck   ' should be ,
        ReplyPollEngineB[6] := serEngineB.rxcheck   ' should be 1 or a 2. Use this returned listener id with PATH or CLOSE commands. 
                                      ' There are a maximum of four listeners available. 
                                      ' Issuing additional LISTEN commands when all four listeners are already established,
                                      '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
        ReplyPollEngineB[7] := serEngineB.rxcheck   ' should be 13 
           
           
           
                
Pub GetPwmInfo        
  
  If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "G" And ReplyPoll[6] == PwmListener And ReplyPoll[7] == 13
    PollError := 0         
    ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
  
    PwmHandle := ReplyPoll[4]  'Store connection identifier.
    
    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data                  
    
    ser.rxflush             
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(PwmHandle)            'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(","))        'Required for a reply
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("200"))      'The desired HTTP response code for the reply.                         
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(","))        'Required for a reply   
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    If CurrentDuty < 10
      ser.str(string("1"))      'Tell the module how many bytes? (or is this digits?) the reply is.
    ElseIf CurrentDuty > 9 AND CurrentDuty < 100                           
      ser.str(string("2"))      'Tell the module how many bytes? (or is this digits?) the reply is.
    Else
      ser.str(string("3"))      'Tell the module how many bytes? (or is this digits?) the reply is. 
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(13)                'End marker                                        
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.dec(CurrentDuty)        'Send 1 byte (which is up to 3 characters (in decimal) (xxx) of data to the module which then sends it to the webpage.                                   
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(13)                'End marker         
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 
    

Pub GetCabooseInfo
  
  If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "G" And ReplyPoll[6] == CabooseWebListener And ReplyPoll[7] == 13
    PollError := 0        
    ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
  
    CabooseWebHandle := ReplyPoll[4]  'Store connection identifier.
    
    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data                  
    
    ser.rxflush             
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("REPLY:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(CabooseWebHandle)            'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(","))        'Required for a reply
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("200"))      'The desired HTTP response code for the reply.                         
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(","))        'Required for a reply   
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("3"))      'Tell the module how many bytes? (or is this digits?) the reply is.         
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(13)                'End marker                                        
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
    If Moves == 0  
      ser.str(string("No "))      'Send Caboose Movement state.
    Elseif Moves == 1
      ser.str(string("Yes"))      'Send Caboose Movement state.                                    
    Else
      ser.str(string(" X "))      'Send Caboose Movement state.
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(13)                'End marker         
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait                            '


Pub GetMotorInfo 
                                  
  If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "P" And ReplyPoll[6] == MotorListener And ReplyPoll[7] == 13     
    PollError := 0   
    ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
  
    MotorHandle := ReplyPoll[4]  'Store connection identifier.
      

    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data                  
    Reply[8] := 0  ' clear data
    
    ser.rxflush             
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string("ARG:"))   'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(MotorHandle)            'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(","))      'Required for a reply
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.str(string(" Speed "))  'The argument name to retrieve.   
    waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
    ser.char(13)                'End marker                                    
    waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Let buffer fill.
    Reply[0] := ser.rxcheck   ' should be 254
    Reply[1] := ser.rxcheck   ' should be =
    Reply[2] := ser.rxcheck   ' should be S
    Reply[3] := ser.rxcheck   ' should be ,
    Reply[4] := ser.rxcheck   ' should be argument value  (this could change) -1 no reply
    Reply[5] := ser.rxcheck   ' should be argument value  (this could change) -1 no reply. This could also be 13
    Reply[6] := ser.rxcheck   ' should be argument value  (this could change) -1 no reply. This could also be 13
    Reply[7] := ser.rxcheck   ' should be argument value  (this could change) -1 no reply. This could also be 13
    Reply[8] := ser.rxcheck   ' should be 13                                                                  
            
    If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," 
      

       
      If Reply[5] == 13  'The end marker
        Reply[5] := 0    'Zero terminate the string so it can be converted to decimal.
        Speed := ser.StrToBase(@byte[@Reply[4]], 10)  ' store and convert string speed in byte variable speed.                                                         
      
      Elseif Reply[6] == 13  'The end marker
        Reply[6] := 0    'Zero terminate the string so it can be converted to decimal.                                              
        Speed := ser.StrToBase(@byte[@Reply[4]], 10)  ' store and convert string speed in byte variable speed.  
               
      Elseif Reply[7] == 13  'The end marker all other conditions the end marker is a byte 7
        Reply[7] := 0        'Zero terminate the string so it can be converted to decimal.   
        Speed := ser.StrToBase(@byte[@Reply[4]], 10)  ' store and convert string speed in byte variable speed.        
      
      Elseif Reply[8] == 13  'The end marker all other conditions the end marker is a byte 8
        Reply[8] := 0        'Zero terminate the string so it can be converted to decimal.
        Speed := 99                                        
      
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait                   
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("REPLY:")) 'Tell the module to send a reply to the poll response.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.char(MotorHandle)            'Should be a number.   An active connection handle; returned by POLL.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("200"))    'The desired HTTP response code for the reply.                         
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("2"))      'Tell the module how many bytes? (or is this digits?) the reply is.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.char(13)                'End marker               
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("OK"))     'Send OK to the webpage.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.char(13)                'End marker         
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 


    Else 

        waitcnt(80_000_000 + cnt) 'Wait for 1s forever. 
        
        ser.char(13)                'End marker         
        
        ser.str(string(27))      'Set ansi command mode in putty.                 
        ser.str(string("[1;5f")) 'Set Position in putty.      
        ser.str(string("ERROR, wrong Speed reply returned.")) 'Tell putty error.                    
        ser.str(string(27))      'Set ansi command mode in putty.      
        ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0
        
        ser.char(13)                'End marker                                          
                                 

Pub GetLedsInfo
            
  If ReplyPoll[0] == 254 And ReplyPoll[1] == "=" And ReplyPoll[2] == "P" And ReplyPoll[6] == LedsListener And ReplyPoll[7] == 13    
    PollError := 0        
    ReplyStatus := 1     'Set the reply to good so it can be shown in putty.
  
    LedsHandle := ReplyPoll[4]  'Store connection identifier.
    

    Reply[0] := 0  ' clear data
    Reply[1] := 0  ' clear data
    Reply[2] := 0  ' clear data
    Reply[3] := 0  ' clear data
    Reply[4] := 0  ' clear data
    Reply[5] := 0  ' clear data
    Reply[6] := 0  ' clear data
    Reply[7] := 0  ' clear data                  
    Reply[8] := 0  ' clear data 
    
    ser.rxflush
    waitcnt(5_000_000 + cnt)    'Wait for 1/80 s   Wait     
    ser.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt)    'Wait for 1/80 s     Wait    
    ser.str(string("ARG:"))     'Retrieve HTTP GET/POST's name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt)    'Wait for 1/80 s     Wait    
    ser.char(LedsHandle)            'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt)    'Wait for 1/80 s     Wait    
    ser.str(string(","))        'Required for a reply
    waitcnt(1_000_000 + cnt)    'Wait for 1/80 s     Wait    
    ser.str(string("io"))       'The argument name to retrieve.   
    waitcnt(1_000_000 + cnt)    'Wait for 1/80 s    Wait    
    ser.char(13)                'End marker                                    
    waitcnt(5_000_000 + cnt)    'Wait for 1/80 s   Let buffer fill.
    Reply[0] := ser.rxcheck   ' should be 254
    Reply[1] := ser.rxcheck   ' should be =
    Reply[2] := ser.rxcheck   ' should be S
    Reply[3] := ser.rxcheck   ' should be ,
    Reply[4] := ser.rxcheck   ' should be p. Argument value  (this could change) -1 no reply
    Reply[5] := ser.rxcheck   ' should be i. Argument value  (this could change) -1 no reply.
    Reply[6] := ser.rxcheck   ' should be n. Argument value  (this could change) -1 no reply. 
    Reply[7] := ser.rxcheck   ' should be #. 0 to 6 Argument value  (this could change) -1 no reply
    Reply[8] := ser.rxcheck   ' should be s. Argument value  (this could change) -1 no reply.
    Reply[9] := ser.rxcheck   ' should be t. Argument value  (this could change) -1 no reply. 
    Reply[10] := ser.rxcheck  ' should be a. Argument value  (this could change) -1 no reply
    Reply[11] := ser.rxcheck  ' should be t. Argument value  (this could change) -1 no reply.
    Reply[12] := ser.rxcheck  ' should be e. Argument value  (this could change) -1 no reply.
    Reply[13] := ser.rxcheck  ' should be #. 1 or 0 Argument value  (this could change) -1 no reply.                                     ' 
    Reply[14] := ser.rxcheck  ' should be 13                                         
    
            
    If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[4] == "p" And Reply[5] == "i" And Reply[6] == "n" And Reply[14] == 13
      
      Case Reply[7]
        "0":                   'The pin number. Headlight
          If Reply[13] == "0"    'The state of the pin
            HeadlightSwitch := 0                                                                                                                         
            PrevHeadlightSwitch := HeadlightSwitch
          Else 
            HeadlightSwitch := 1
            PrevHeadlightSwitch := HeadlightSwitch
                                                                                                                         
        "1":  'The pin number.  Rearlight
          If Reply[13] == "0"    'The state of the pin
            RearlightSwitch := 0            
            PrevRearlightSwitch := RearlightSwitch                                                     
          Else 
            RearlightSwitch := 1
            PrevRearlightSwitch := RearlightSwitch
                                              
        "2":  'The pin number.  Smoker
          If Reply[13] == "0"    'The state of the pin
            SmokerSwitch := 0
          Else 
            SmokerSwitch := 1              
        
        "3":  'The pin number.  
          If Reply[13] == "0"    'The state of the pin
            Whistle := 0 
          Else 
            Whistle := 1  
        
        "4":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            Bell := 0 
          Else 
            Bell := 1 
        
        "5":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            StackTalk := 0 
          Else 
            StackTalk := 1  
            
        "6":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            Reverse := 0 
          Else 
            Reverse := 1                                      
        
        "7":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            EmergencyStop := 0 
          Else 
            EmergencyStop := 1            
        
        "8":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            FastStart := 0 
          Else 
            FastStart := 1
        
        "9":  'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            TimerSwitch := 0 
          Else 
            TimerSwitch := 1
            
        "A": 'The pin number.
          If Reply[13] == "0"    'The state of the pin                      
            EotStop := 0 
          Else 
            EotStop := 1
          
                              
        Other:

          waitcnt(80_000_000 + cnt) 'Wait for 1s forever. 
          
          ser.char(13)                'End marker         
          
          ser.str(string(27))      'Set ansi command mode in putty.                 
          ser.str(string("[1;5f")) 'Set Position in putty.      
          ser.str(string("ERROR, wrong Switches pin returned.")) 'Tell putty error.                    
          ser.str(string(27))      'Set ansi command mode in putty.      
          ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0   

          ser.char(13)                'End marker         

      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait                            
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("REPLY:")) 'Tell the module to send a reply to the poll response.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.char(LedsHandle)            'Should be a number.   An active connection handle; returned by POLL.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("200"))    'The desired HTTP response code for the reply.                         
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string(","))      'Required for a reply
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("2"))      'Tell the module how many bytes? (or is this digits?) the reply is.
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.char(13)                'End marker               
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
      ser.str(string("OK"))     'Send OK to the webpage.                                                  
      waitcnt(1_000_000 + cnt) 'Wait for 1/80 Wait
      ser.char(13)                'End marker         
      waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 
      

Pub ConnectToCaboose

 
  If CabooseListener == 0       
    ReplyEngineB[0] := 0  ' clear data
    ReplyEngineB[1] := 0  ' clear data
    ReplyEngineB[2] := 0  ' clear data
    ReplyEngineB[3] := 0  ' clear data
    ReplyEngineB[4] := 0  ' clear data
    ReplyEngineB[5] := 0  ' clear data
    ReplyEngineB[6] := 0  ' clear data
    ReplyEngineB[7] := 0  ' clear data
    
    serEngineB.rxflush                     
    waitcnt(5_000_000 + cnt)    'wait 1/16 second.
    serEngineB.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt)    'wait 1/16 second.
    serEngineB.str(string("CONNECT:caboose.railroad.local,80"))    'Tell the module to connect to a specific address.
    waitcnt(1_000_000 + cnt)    'wait 1/16 second.
    serEngineB.char(13)                'End marker     

    waitcnt(5_000_000 + cnt)    'wait 1/16 second. Testing shorter wait times for connect.
    ReplyEngineB[0] := serEngineB.rxcheck     ' should be 254
    ReplyEngineB[1] := serEngineB.rxcheck     ' should be =
    ReplyEngineB[2] := serEngineB.rxcheck     ' should be S
    ReplyEngineB[3] := serEngineB.rxcheck     ' should be ,
    ReplyEngineB[4] := serEngineB.rxcheck     ' should be 5, 6 this could change. Use the returned connection handle with PATH, CLOSE, SEND, or RECV commands.
                                ' There are a maximum of four connections available; shared between successfulCONNECT requests and active listener connections.
                                ' Issuing additional CONNECT commands when all four connections are already established will result in a
                                ' response of b = E,5 E (NO_FREE_CONNECTION) until a CLOSE command is used to free a connection.
    ReplyEngineB[5] := serEngineB.rxcheck     ' should be 13  
    
      If ReplyEngineB[0] == 254 And ReplyEngineB[1] == "=" And ReplyEngineB[2] == "S" And ReplyEngineB[3] == "," And ReplyEngineB[5] == 13 

        CabooseListener := ReplyEngineB[4]   
        
        ConnectErrorEngineB := 0
        
      Else

          ConnectErrorEngineB := 1
          CabooseListener := 0

Pub RequestCabooseData

    If CabooseListener <> 0
        ReplyEngineB[0] := 0  ' clear data
        ReplyEngineB[1] := 0  ' clear data
        ReplyEngineB[2] := 0  ' clear data
        ReplyEngineB[3] := 0  ' clear data
        ReplyEngineB[4] := 0  ' clear data
        ReplyEngineB[5] := 0  ' clear data
       
        serEngineB.rxflush                     
        waitcnt(5_000_000 + cnt)    'wait 1/16 second.
        serEngineB.char(254)               'Begin marker
        waitcnt(1_000_000 + cnt)    'wait 1/16 second.
        serEngineB.str(string("SEND:"))    'Retrieve the path associated with a connection handle or listener id.
        waitcnt(1_000_000 + cnt)    'wait 1/16 second.
        serEngineB.char(CabooseListener)    'An active connection handle; returned by POLL                            
        waitcnt(1_000_000 + cnt)    'wait 1/16 second.
        serEngineB.str(string(","))      'Tell the module how many bytes? (or is this digits?) the send is.
        waitcnt(1_000_000 + cnt)    'wait 1/16 second.
        'serEngineB.str(string("87"))      'Tell the module how many bytes? (or is this digits?) the send is.
        serEngineB.str(string("92"))      'Tell the module how many bytes? (or is this digits?) the send is.
        waitcnt(1_000_000 + cnt)    'wait 1/16 second.
        serEngineB.char(13)                'End marker    
        waitcnt(5_000_000 + cnt)    'Wait for 1/16 s  Let buffer fill.    

'-------------------------------Begin HTTP Client Request Packet---------------------------------------------

        serEngineB.str(string("GET /cabooze HTTP/1.1"))        '21 
        serEngineB.char(13)                '\r                   1
        serEngineB.char(10)                '\n                   1
        serEngineB.str(string("Host: caboose.railroad.local")) '28
        'serEngineB.str(string("Host: pcb2.railraod.local"))    '25                                                        
        serEngineB.char(13)                '\r                   1
        serEngineB.char(10)                '\n                   1
        serEngineB.str(string("Connection: keep-alive"))       '22
        'serEngineB.str(string("Connection: close"))            '17
        serEngineB.char(13)                '\r                   1
        serEngineB.char(10)
        'serEngineB.str(string("Keep-Alive: timeout=5, max=1000"))       '22
        'serEngineB.char(13)                '\r                   1
        'serEngineB.char(10)                '\n                   1                '\n                   1
        serEngineB.str(string("Accept: */*"))                  '11           
        serEngineB.char(13)                '\r                   1
        serEngineB.char(10)                '\n                   1
        serEngineB.char(13)                '\r                   1
        serEngineB.char(10)                '\n                   1    92  or 89   or  87
    
        waitcnt(1_000_000 + cnt)  'Wait for 1/16 s 

        serEngineB.char(13)                'End marker         
        waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait 

Pub GetDataFromCaboose
 If CabooseListener <> 0
  If ReplyPollEngineB[0] == 254 And ReplyPollEngineB[1] == "=" And ReplyPollEngineB[2] == "D" 'And ReplyPollEngineB[4] == "5"' And ReplyPollEngineB[9]= 13
    PollErrorEngineB := 0
      
    If ReplyPollEngineB[7] <> 13               'Not the end marker
      ReplyPollEngineB[8] := serEngineB.rxcheck       'should be number of bytes being sent, could be the end marker.     
      If ReplyPollEngineB[8] <> 13             'Not the end marker
        ReplyPollEngineB[9] := serEngineB.rxcheck   'should be number of bytes being sent, could be the end marker.  
        If ReplyPollEngineB[9] == 13         'The end marker all other conditions the end marker is a byte 7
          ReplyPollEngineB[9] := 0           'Zero terminate the string so it can be converted to decimal.
      Else
        ReplyPollEngineB[8] := 0         'Zero terminate the string so it can be converted to decimal.
    Else
      ReplyPollEngineB[7] := 0           'Zero terminate the string so it can be converted to decimal.               

    ReplyStatusEngineB := 1       'Set the reply to good so it can be shown in putty.
             
    CabooseHandle := ReplyPollEngineB[4]     'Store connection identifier.
   
    DataCount1 := serEngineB.StrToBase(@long[@ReplyPollEngineB[6]], 10)  ' store and convert string speed in byte variable speed.  
    
    ReplyEngineB[0] := 0  ' clear data
    ReplyEngineB[1] := 0  ' clear data
    ReplyEngineB[2] := 0  ' clear data
    ReplyEngineB[3] := 0  ' clear data
    ReplyEngineB[4] := 0  ' clear data
    ReplyEngineB[5] := 0  ' clear data
    ReplyEngineB[6] := 0  ' clear data
    ReplyEngineB[7] := 0  ' clear data
               '   
    serEngineB.rxflush             
    waitcnt(5_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.str(string("RECV:"))    'Retrieve HTTP GET/POSTâ€™s name argument (in query or body) on connection handle.
    waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.char(CabooseHandle)  'An active connection handle; returned by POLL
    waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.str(string(","))        'Required for a reply
    waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.dec(DataCount1)          'The desired length of data to recieve.                         
    waitcnt(1_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
    serEngineB.char(13)                'End marker                                        
    waitcnt(5_000_000 + cnt) 'Wait for 1/16 s  Let buffer fill.
      
    ReplyEngineB[0] := serEngineB.rxcheck   ' should be 254
    ReplyEngineB[1] := serEngineB.rxcheck   ' should be =
    ReplyEngineB[2] := serEngineB.rxcheck   ' should be S
    ReplyEngineB[3] := serEngineB.rxcheck   ' should be ,
    ReplyEngineB[4] := serEngineB.rxcheck   ' should be number of bytes being sent.            
    ReplyEngineB[5] := serEngineB.rxcheck   ' should be number of bytes being sent, could be the end marker.
    
    If ReplyEngineB[5] <> 13         'Not the end marker
      ReplyEngineB[6] := serEngineB.rxcheck ' should be number of bytes being sent, could be the end marker.     
      If ReplyEngineB[6] <> 13       'Not the end marker
        ReplyEngineB[7] := serEngineB.rxcheck ' should be number of bytes being sent, could be the end marker.  
        If ReplyEngineB[7] == 13     'The end marker all other conditions the end marker is a byte 7
          ReplyEngineB[7] := 0       'Zero terminate the string so it can be converted to decimal.
     
      Else
        ReplyEngineB[6] := 0         'Zero terminate the string so it can be converted to decimal.
    Else
      ReplyEngineB[5] := 0           'Zero terminate the string so it can be converted to decimal. 
    
    DataCount2 := serEngineB.StrToBase(@long[@ReplyEngineB[4]], 10)  ' store and convert string speed in byte variable speed.            
    
    If ReplyEngineB[0] == 254 And ReplyEngineB[1] == "=" And ReplyEngineB[2] == "S" And ReplyEngineB[3] == ","
                    
      ReplyStatusEngineB := 1       'Set the reply to good so it can be shown in putty.                               
                                            
      If DataCount1 == DataCount2
        DataCount2 := 0
        bytenumber := 0
        Repeat Datacount1
          Data1[bytenumber] := serEngineB.CharIn        
          bytenumber++
                       
        TiltX := 0
        TiltY := 0
        Moves := -1
        
        bytenumber := 84
        Repeat DataCount1 - 84
            
          
          If Data1[bytenumber - 6] == "X"
            If Data1[bytenumber - 1] == ">"                       
              TiltX := serEngineB.StrToBase(@byte[@Data1][91], 10)
                                   
              TiltY := serEngineB.StrToBase(@byte[@Data1][104], 10)
              
              Moves := serEngineB.StrToBase(@byte[@Data1][117], 10)
              
              
              ReplyStatus := 1
                            
          bytenumber++
        
        Xchange2 := Xchange1 - TiltX 
        
        Xchange1 := TiltX
        
        Ychange2 := Ychange1 - TiltY 
        
        Ychange1 := TiltY
        
        AxisChange := ||Xchange2 + ||Ychange2
        
        If TiltX == 0 or TiltY == 0 or AxisChange < 9 or Moves == 0
          CabooseMove := 0 
        Else
          CabooseMove := 1       
        
        CabooseConnect := 0

      
Pub CloseCaboose

    If CabooseListener <> 0

    
      ReplyEngineB[0] := 0  ' clear data
      ReplyEngineB[1] := 0  ' clear data
      ReplyEngineB[2] := 0  ' clear data
      ReplyEngineB[3] := 0  ' clear data
      ReplyEngineB[4] := 0  ' clear data
      ReplyEngineB[5] := 0  ' clear data
      ReplyEngineB[6] := 0  ' clear data
      ReplyEngineB[7] := 0  ' clear data
      
      serEngineB.rxflush                     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))    'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(CabooseListener)    'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      ReplyEngineB[0] := serEngineB.rxcheck     ' should be 254
      ReplyEngineB[1] := serEngineB.rxcheck     ' should be =
      ReplyEngineB[2] := serEngineB.rxcheck     ' should be S
      ReplyEngineB[3] := serEngineB.rxcheck     ' should be ,
      ReplyEngineB[4] := serEngineB.rxcheck     ' should be 0 if success. this could change. Use the returned connection handle with PATH, CLOSE, SEND, or RECV commands.
                                  ' There are a maximum of four connections available; shared between successfulCONNECT requests and active listener connections.
                                  ' Issuing additional CONNECT commands when all four connections are already established will result in a
                                  ' response of b = E,5 E (NO_FREE_CONNECTION) until a CLOSE command is used to free a connection.
      ReplyEngineB[5] := serEngineB.rxcheck     ' should be 13  
            
      
      If ReplyEngineB[0] == 254 And ReplyEngineB[1] == "=" And ReplyEngineB[2] == "S" And ReplyEngineB[3] == "," And ReplyEngineB[4] == "0" And ReplyEngineB[5] == 13 
  
        CabooseListener := 0

Pub CloseCabooseHandle
        
    ReplyEngineB[0] := 0  ' clear data
    ReplyEngineB[1] := 0  ' clear data
    ReplyEngineB[2] := 0  ' clear data
    ReplyEngineB[3] := 0  ' clear data
    ReplyEngineB[4] := 0  ' clear data
    ReplyEngineB[5] := 0  ' clear data
    ReplyEngineB[6] := 0  ' clear data
    ReplyEngineB[7] := 0  ' clear data
    
    serEngineB.rxflush                     
    waitcnt(5_000_000 + cnt)    'wait 1/16 second.
    serEngineB.char(254)               'Begin marker
    waitcnt(1_000_000 + cnt)    'wait 1/16 second.
    serEngineB.str(string("CLOSE:"))    'Tell the module to connect to a specific address.
    waitcnt(1_000_000 + cnt)    'wait 1/16 second.
    serEngineB.char(CabooseHandle)    'An active connection handle; returned by POLL   
    waitcnt(1_000_000 + cnt)    'wait 1/16 second.
    serEngineB.char(13)                'End marker     
    waitcnt(5_000_000 + cnt)    'wait 1/16 second.
    CabooseHandle := 0

Pub StartListeners

  StopListeners

  StartMotorListener
  StartLedsListener  
  StartPwmListener  
  StartCabooseWebListener  
  
  ser.char(13)                'End marker         
  
  ser.str(string(27))           'Set ansi command mode in putty.                 
  ser.str(string("[2;13f"))      'Set Position in putty.      
  ser.str(string("Listeners Started.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0        

  ser.char(13)                'End marker         

Pub StopListeners
   
  StopMotorListener
  StopLedsListener  
  StopPwmListener  
  StopCabooseWebListener  

  CloseHandles
   
  ser.char(13)                'End marker         
  
  ser.str(string(27))           'Set ansi command mode in putty.                 
  ser.str(string("[2;13f"))      'Set Position in putty.      
  ser.str(string("Listeners Stopped.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0        

  ser.char(13)                'End marker         
    
  
Pub CloseHandles

  CloseMotorHandle
  CloseLedsHandle
  ClosePwmHandle
  CloseCabooseWebHandle  
  
  ser.char(13)                'End marker         
  
  ser.str(string(27))           'Set ansi command mode in putty.                 
  ser.str(string("[2;23f"))      'Set Position in putty.      
  ser.str(string("Handles Clozed.")) 'Tell putty command mode is on.                       
  ser.str(string(27))           'Set ansi command mode in putty.      
  ser.str(string("[1;1f"))      'Set Position in putty to put cursor at 0        

  ser.char(13)                'End marker         

Pub StartMotorListener
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait                     
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/motor"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 1   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[5] == 13 
    MotorListener := Reply[4]
    ListenError := 0  
  Else
 
      
      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;5f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Web speed listener returned.")) 'Tell putty error.                       
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  

      ser.char(13)                'End marker         

      ListenError := 1  
      MotorListener := 0

Pub CloseMotorHandle

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(MotorHandle)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  MotorHandle := 0

Pub StopMotorListener
    
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(MotorListener)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  MotorListener := 0

Pub StartLedsListener

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
    
  ser.rxflush 
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/leds"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 2   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
                            
                            
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[5] == 13 
    LedsListener := Reply[4]
    ListenError := 0 
  Else

      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;5f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Web switch listener returned.")) 'Tell putty error.                    
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;5f")) 'Set Position in putty to put cursor at 0   
      
      ser.char(13)                'End marker         
      
      ListenError := 1
      LedsListener := 0

Pub CloseLedsHandle

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(LedsHandle)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  LedsHandle := 0                          

Pub StopLedsListener

  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(LedsListener)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  LedsListener := 0 

Pub StartPwmListener

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/PwmValue"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[5] == 13 
  
    PwmListener := Reply[4]  
    ListenError := 0
  
  Else

      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;5f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Pwm listener returned.")) 'Tell putty error.                       
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0  

      ser.char(13)                'End marker         

      ListenError := 1
      PwmListener := 0

Pub ClosePwmHandle

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(PwmHandle)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  PwmHandle := 0

Pub StopPwmListener
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(PwmListener)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  PwmListener := 0

Pub StartCabooseWebListener

  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data

  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("LISTEN:HTTP,/CabooseWeb"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 4   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  
  If Reply[0] == 254 And Reply[1] == "=" And Reply[2] == "S" And Reply[3] == "," And Reply[5] == 13 
  
    CabooseWebListener := Reply[4] 
    ListenError := 0 
  
  Else

      ser.char(13)                'End marker         
      
      ser.str(string(27))      'Set ansi command mode in putty.                 
      ser.str(string("[1;5f")) 'Set Position in putty.      
      ser.str(string("ERROR, wrong Cab005E movement listener returned.")) 'Tell putty error.                       
      ser.str(string(27))      'Set ansi command mode in putty.      
      ser.str(string("[1;1f")) 'Set Position in putty to put cursor at 0
      
      ser.char(13)                'End marker         
      
      ListenError := 1 
      CabooseWebListener := 0

Pub CloseCabooseWebHandle
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(CabooseWebHandle)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  CabooseWebHandle := 0

Pub StopCabooseWebListener
  
  Reply[0] := 0  ' clear data
  Reply[1] := 0  ' clear data
  Reply[2] := 0  ' clear data
  Reply[3] := 0  ' clear data
  Reply[4] := 0  ' clear data
  Reply[5] := 0  ' clear data
  
  ser.rxflush                     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(254)  'Begin marker
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.str(string("CLOSE:"))   'Tell the module to listen to a specific path on http port 80.
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait
  ser.char(CabooseWebListener)         'An active connection handle; returned by POLL
  waitcnt(1_000_000 + cnt) 'Wait for 1/80 s   Wait                              '
  ser.char(13)    'End marker     
  waitcnt(5_000_000 + cnt) 'Wait for 1/80 s  Let buffer fill.
  Reply[0] := ser.rxcheck   ' should be 254
  Reply[1] := ser.rxcheck   ' should be =
  Reply[2] := ser.rxcheck   ' should be S
  Reply[3] := ser.rxcheck   ' should be ,
  Reply[4] := ser.rxcheck   ' should be 3   Use this returned listener id with PATH or CLOSE commands. 
                            ' There are a maximum of four listeners available. 
                            ' Issuing additional LISTEN commands when all four listeners are already established,
                            '  will result in a response of b = E,4 E (NO_FREE_LISTENER) until a CLOSE command is used to free a listener.
  Reply[5] := ser.rxcheck   ' should be 13
  CabooseWebListener := 0


Pub CloseAllConnections
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(PwmHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      PwmHandle := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(MotorHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      MotorHandle := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(LedsHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      LedsHandle := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(CabooseWebHandle)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      CabooseWebHandle := 0
      
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(MotorListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      MotorListener := 0
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(LedsListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      LedsListener := 0 
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(PwmListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      PwmListener := 0

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(CabooseWebListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.      
      CabooseWebListener := 0
      
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("1"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("2"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("3"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("4"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("5"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("6"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("7"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.str(string("8"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      ser.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

 
Pub CloseAllConnectionsEngineB
      
      'CloseCaboose
      {
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(CabooseHandle)     'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      }
      
      CabooseHandle := 0 

      
      {
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(CabooseListener)         'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      }
      
      CabooseListener := 0 

            
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("1"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("2"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("3"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("4"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("5"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("6"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
      
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("7"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.

      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(254)               'Begin marker
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("CLOSE:"))   'Tell the module to connect to a specific address.
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.str(string("8"))        'An active connection handle; returned by POLL   
      waitcnt(1_000_000 + cnt)    'wait 1/16 second.
      serEngineB.char(13)                'End marker     
      waitcnt(5_000_000 + cnt)    'wait 1/16 second.
         


DAT
  PropStartup       byte  0
  
  SecondsModifier   byte  51
  TimedSpeed        byte  99
  Started           byte  0
  MinuteChime       byte  0
  HourChime         byte  0
  Micro             byte  0
  
  TimerSwitch       byte  0
  FiveMinute        byte  3
  QuarterMinute     byte  0
  
  Seconds           long  30
  Minutes           long  38
  Hours             long  1
  CabooseConnect    long  0
  
  RestartError      byte  0
  PollError         byte  0
  PollErrorEngineB  byte  0
  ListenError       byte  0
  ConnectError      byte  0 
  ConnectErrorEngineB byte  0 
  HandleError       byte  0

  Moves             long  -1
  CabooseMove       byte  0
  
  AxisChange        long  0
  
  Xchange1          long  0
  Xchange2          long  0
  
  Ychange1          long  0
  Ychange2          long  0
  
  TiltX             long  0, 0
  TiltY             long  0, 0
  
  ClientCounter     long  0
  PollCounter       long  0
  PollCounterEngineB long  0
  
  Datacount1        long  0, 0
  Datacount2        long  0, 0
  
  Bytenumber        long  0
  
  LoopCounter1      long  0
  
  Data0             byte  " "[100]  'If the data sent is huge this could overload ram.                   
  Data1             byte  " "[100]  'If the data sent is huge this could overload ram.
  Data2             byte  " "[100]  'If the data sent is huge this could overload ram.

  
  EmergencyStop     byte  1   'this byte holds the emergency stop of the motor
  Reverse           byte  0   'this byte holds the direction of the motor
  Whistle           byte  0   'this byte holds the whisle on or off
  Bell              byte  0   'this byte holds the bell on or off
  StackTalk         byte  0   'this byte holds the stacktalk on or off
  Speed             long  0
  CurrentDuty       long  0
  CurrentSpeed      long  0
  FastStart         byte  1
  HeadlightSwitch   byte  1
  RearlightSwitch   byte  1
  PrevHeadlightSwitch byte  1
  PrevRearlightSwitch byte  1
  SmokerSwitch      byte  0
  EotStop           byte  1

  TrainMoving       byte  0
  TrainMovingCount  byte  0
  
  'Byte[#]               '0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 18 20
  Reply             byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  ReplyEngineB      byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  ReplyPoll         byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
  ReplyPollEngineB  byte  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////      
}}   